﻿--------------------------------------------------------------------------------------------------------
--                                             Localized global                                       --
--------------------------------------------------------------------------------------------------------
local _G = getfenv(0)

--------------------------------------------------------------------------------------------------------
--                                             AceAddon init                                          --
--------------------------------------------------------------------------------------------------------
local MODNAME = "Broker_Wintergrasper"
local addon = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceEvent-3.0", "AceTimer-3.0")
_G.Broker_Wintergrasper = addon

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

--------------------------------------------------------------------------------------------------------
--                           Broker_Wintergrasper variables and defaults                              --
--------------------------------------------------------------------------------------------------------
local FACTION_UNKNOWN = 0
local FACTION_ALLIANCE = 1
local FACTION_HORDE = 2
local FACTION_BATTLE = 3

local faction = FACTION_UNKNOWN
local nextBattle = nil
local old_faction = FACTION_UNKNOWN
local old_nextBattle = nil

local profileDB
local realmDB
local DATABASE_DEFAULTS = {
	profile = {
		showSeconds = false,
		tellSelf = true,
		tellGuild = true,
		tellParty = false,
		tellRaid = false,
		timeNow = false,
		timeFive = false,
		timeFifteen = true,
		timeThirty = false,
		soundAlert = true,
		autoQueue = false,
	},
	realm = {
		nextBattleTime = nil,
		nextBattleFaction = nil,
	},
}

local canUpdateHere = true

--------------------------------------------------------------------------------------------------------
--                              Broker_Wintergrasper options pannel                                   --
--------------------------------------------------------------------------------------------------------
addon.options = {
	type = "group",
	name = "Broker Wintergrasper",
	args = {
		general = {
			order = 1,
			type = "group",
			name = L["General Settings"],
			cmdInline = true,
			args = {
				seconds = {
					order = 1,
					type = "toggle",
					name = L["Show seconds"],
					desc = L["Toggle to show seconds."],
					get = function()
						return profileDB.showSeconds
					end,
					set = function(key, value)
						profileDB.showSeconds = value
					end,
				},
				sound = {
					order = 2,
					type = "toggle",
					name = L["Sound alert"],
					desc = L["Toggle to play a sound alert besides the chat warning."],
					get = function()
						return profileDB.soundAlert
					end,
					set = function(key, value)
						profileDB.soundAlert = value
					end,
				},
				queue = {
					order = 3,
					type = "toggle",
					name = L["Auto queue"],
					desc = L["Toggle to automatically atemp to queue 15 minutes before battle."],
					get = function()
						return profileDB.autoQueue
					end,
					set = function(key, value)
						profileDB.autoQueue = value
					end,
				},
				announceTo = {
					order = 2,
					type = 'group',
					name = L["Announce to"],
					desc = L["Choose whom to announce when battle will start."],
					args = {
						self = {
							order = 1,
							type = 'toggle',
							name = L["Self"],
							desc = L["Warn self on upcoming battle."],
							get = function() return profileDB.tellSelf end,
							set = function(key, value) profileDB.tellSelf = value end,
						},
						guild = {
							order = 2,
							type = 'toggle',
							name = L["Guild"],
							desc = L["Warn guild on upcoming battle."],
							get = function() return profileDB.tellGuild end,
							set = function(key, value) profileDB.tellGuild = value end,
						},
						party = {
							order = 3,
							type = 'toggle',
							name = L["Party"],
							desc = L["Warn party on upcoming battle."],
							get = function() return profileDB.tellParty end,
							set = function(key, value) profileDB.tellParty = value end,
						},
						raid = {
							order = 4,
							type = 'toggle',
							name = L["Raid"],
							desc = L["Warn raid on upcoming battle."],
							get = function() return profileDB.tellRaid end,
							set = function(key, value) profileDB.tellRaid = value end,
						},
					},
				},
				announceTime = {
					order = 3,
					type = 'group',
					name = L["Announce time"],
					desc = L["Choose when to announce the upcoming battle."],
					args = {
						now= {
							order = 1,
							type = 'toggle',
							name = L["Start"],
							desc = L["Warn at battle start."],
							get = function() return profileDB.timeNow end,
							set = function(key, value) profileDB.timeNow = value end,
						},
						five= {
							order = 2,
							type = 'toggle',
							name = "5" .. L[" minutes."],
							desc = L["Warn "] .. "5" .. L[" minutes before battle."],
							get = function() return profileDB.timeFive end,
							set = function(key, value) profileDB.timeFive= value end,
						},
						fifteen= {
							order = 3,
							type = 'toggle',
							name = "15" .. L[" minutes."],
							desc = L["Warn "] .. "15" .. L[" minutes before battle."],
							get = function() return profileDB.timeFifteen end,
							set = function(key, value) profileDB.timeFifteen = value end,
						},
						thirty = {
							order = 4,
							type = 'toggle',
							name = "30" .. L[" minutes."],
							desc = L["Warn "] .. "30" .. L[" minutes before battle."],
							get = function() return profileDB.timeThirty end,
							set = function(key, value) profileDB.timeThirty = value end,
						},
					},
				},
			},
		},
	},
}

function addon:SetupOptions()
	addon.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	addon.options.args.profile.order = -2

	AceConfig:RegisterOptionsTable(MODNAME, addon.options, nil)

	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions(MODNAME, L["Profiles"], MODNAME, "profile")
end

--------------------------------------------------------------------------------------------------------
--                                       Broker_Wintergrasper core                                    --
--------------------------------------------------------------------------------------------------------
function addon:OnInitialize()
	self.db = AceDB:New("Broker_WintergrasperDB", DATABASE_DEFAULTS, true)
	if not self.db then
		_G.print("Error: Database not loaded correctly.  Please exit out of WoW and delete Broker_Wintergrasper.lua found in: \\World of Warcraft\\WTF\\Account\\<Account Name>>\\SavedVariables\\")
	end

	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	profileDB = self.db.profile
	realmDB = self.db.realm
	self:SetupOptions()

	self:UpdateFaction()
	if faction == FACTION_UNKNOWN then
		if realmDB.nextBattleTime then
			if realmDB.nextBattleTime > _G.time() then
				nextBattle = realmDB.nextBattleTime
				faction = realmDB.nextBattleFaction
			else
				nextBattle = nil
				faction = FACTION_UNKNOWN
			end
		end
	end
	self:UpdateIcon()

	self:RegisterEvent("ZONE_CHANGED_NEW_AREA","ScheduleUpdate")
end

function addon:OnEnable()
	self:ScheduleRepeatingTimer("MainUpdate", 1)
end

-- LDB object
addon.obj = ldb:NewDataObject(MODNAME, {
	type = "data source",
	label = L["Wintergrasp"],
	text = "",
	icon = "Interface\\Icons\\Inv_essenceofwintergrasp",
	OnClick = function(frame, msg)
		if msg == "LeftButton" then
			addon:OnClick()
		elseif msg == "RightButton" then
			addon:ShowConfig()
		end
	end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine(MODNAME.." ".._G.GetAddOnMetadata(MODNAME, "Version"))
		tooltip:AddLine(L["Shift-hint"])
		tooltip:AddLine(L["Right-hint"])
	end,
})

-- Main update function
function addon:MainUpdate()
	if nextBattle == nil then
		self:UpdateTime()
		if nextBattle == nil then
			self.obj.text = "|cffffff00" .. addon:GetLDBText() .. "|r"
			return
		end
	end

	if faction == FACTION_UNKNOWN or (faction == FACTION_BATTLE  and (nextBattle - _G.time() < 8700))then
		if canUpdateHere then
			self:DoUpdate()
		end
	end

	local seconds = nextBattle - _G.time()
	if (seconds > 0) then
		if seconds == 1800 and profileDB.timeThirty then
			self:Warn(30)
		elseif seconds == 900 and profileDB.timeFifteen then
			self:Warn(15)
		elseif seconds == 300 and profileDB.timeFive then
			self:Warn(5)
		end
		if seconds == 890 and _G.UnitLevel("player") >= 70 and profileDB.autoQueue then
			_G.BattlefieldMgrQueueRequest(1)
		end
	elseif (seconds == 0) then
		if profileDB.timeNow then
			self:Warn(0)
		end
		old_faction = faction
		faction = FACTION_BATTLE
		self:UpdateIcon()
		old_nextBattle = nextBattle
		nextBattle = nextBattle + 9000
	else
		faction = FACTION_UNKNOWN
		nextBattle = nil
	end

	self.obj.text = "|cffffff00" .. addon:GetLDBText() .. "|r"
end

--------------------------------------------------------------------------------------------------------
--                                   Broker_Wintergrasper functions                                   --
--------------------------------------------------------------------------------------------------------

-- Called after profile changed
function addon:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile
end

-- Open config window
function addon:ShowConfig()
	_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile)
	_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end

-- On click event handler
function addon:OnClick()
	if _G.IsShiftKeyDown() then
		if _G.ChatFrame1EditBox:IsVisible() then
			local message = self:GetAnnounceText()
			_G.ChatFrame1EditBox:Insert(message)
		end
	end
end

-- Allow a new update
function addon:ScheduleUpdate()
	canUpdateHere = true
end

-- Get fresh data
function addon:DoUpdate()
	self:UpdateFaction()
	self:UpdateIcon()

	if nextBattle == nil then
		self:UpdateTime()
	end

	addon:SaveData()
end

-- Update time.
function addon:UpdateTime()
	local pvpID, zone, isActive, canQueue, startTime, canEnter = _G.GetWorldPVPAreaInfo(1);
	if startTime > 5 then
		nextBattle = _G.time() + startTime
	end
end

-- Search for WG status and update faction.
function addon:UpdateFaction()
	local pvpID, zone, isActive, canQueue, startTime, canEnter = _G.GetWorldPVPAreaInfo(1);
	if startTime > 0 then
		if isActive then
			if not (faction == FACTION_BATTLE) then
				old_faction = faction
				faction = FACTION_BATTLE
			end
		else
			if (faction == FACTION_BATTLE) then
				self:ScheduleTimer("DoUpdate", 3)
			end
			if not _G.WorldMapFrame:IsVisible() then
				local found = false
				local previousMapID = _G.GetCurrentMapAreaID()

				_G.SetMapByID(501)
				local n = _G.GetNumMapLandmarks()
				if n > 0 then
					local i = 0
					while (i < n) and (not found) do
						i = i + 1
						local name, _, textureIndex = _G.GetMapLandmarkInfo(i)
						if name == L["Fortress Graveyard"] then
							found = true
							if textureIndex == 15 then
								faction = FACTION_ALLIANCE
							elseif textureIndex == 13 then
								faction = FACTION_HORDE
							end
						end
					end
				end
				_G.SetMapByID(previousMapID)
				if found then
					self:UpdateTime()
				else
					canUpdateHere = false
					faction = FACTION_UNKNOWN
				end
			else
				faction = FACTION_UNKNOWN
			end
		end
	end
end

-- Save time and faction in config files
function addon:SaveData()
	realmDB.nextBattleTime = nextBattle
	realmDB.nextBattleFaction = faction
end

-- Update the icon based on faction.
function addon:UpdateIcon()
	if faction == FACTION_UNKNOWN then
		self.obj.icon = "Interface\\Icons\\Inv_essenceofwintergrasp"
	elseif faction == FACTION_ALLIANCE then
		self.obj.icon = "Interface\\pvpframe\\pvp-currency-alliance"
	elseif faction == FACTION_HORDE then
		self.obj.icon = "Interface\\pvpframe\\pvp-currency-horde"
	elseif faction == FACTION_BATTLE then
		self.obj.icon = "Interface\\Icons\\Ability_DualWield"
	end
end

-- Return LDB display text
function addon:GetLDBText()
	local timeText
	if faction == FACTION_BATTLE then
		if old_faction == FACTION_ALLIANCE then
			if "Alliance" == _G.UnitFactionGroup("player") then
				timeText =  L["Defensive"]
			else
				timeText =  L["Offensive"]
			end
		elseif old_faction == FACTION_HORDE then
			if "Horde" == _G.UnitFactionGroup("player") then
				timeText =  L["Defensive"]
			else
				timeText =  L["Offensive"]
			end
		else
			timeText = L["Battle"]
		end
		if old_nextBattle then
			timeText = timeText .. " " .. _G.string.sub(self:GetTimeText(_G.time() - old_nextBattle), 4)
		end
	else
		if not nextBattle then
			timeText = L["Unknown"]
		elseif nextBattle then
			local seconds = nextBattle - _G.time()
			if (seconds >= 0) then
				timeText = self:GetTimeText(seconds)
			else
				timeText = L["Unknown"]
			end
		end
	end
	return timeText
end

-- Return Wintergasp wait time in "h:m:s"
function addon:GetTimeText(seconds)
	local timeText = ""
	local h = _G.math.floor(seconds / 3600)
	local m = _G.math.floor((seconds % 3600) / 60)
	local s = seconds % 60
	if profileDB.showSeconds then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	elseif seconds > 59 then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m
		elseif m > 0 then
			timeText = timeText .. "0" .. m
		else
			timeText = timeText .. "00"
		end
	else
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	end
	return timeText
end

-- Return battle type text
function addon:GetBattleTypeText()
	if faction == FACTION_UNKNOWN then
		return L["Next battle in"]
	elseif faction == FACTION_BATTLE then
		return L["The battle for Wintergrasp is underway."]
	elseif faction == FACTION_ALLIANCE then
		if "Alliance" == _G.UnitFactionGroup("player") then
			return L["Defensive battle in"]
		else
			return L["Offensive battle in"]
		end
	else
		if "Horde" == _G.UnitFactionGroup("player") then
			return L["Defensive battle in"]
		else
			return L["Offensive battle in"]
		end
	end
end

-- Create text about Wintergrasp status
function addon:GetAnnounceText()
	local message = "[Broker_Wintergrasper] " .. self:GetBattleTypeText()
	if not (faction == FACTION_BATTLE) then
		if nextBattle then
			message = message .. " " .. self:GetTimeText(nextBattle - _G.time())
		else
			message = message .. " " .. L["Unknown"]
		end
	end
	return message
end

-- Visual and audio warning for upcoming battle
function addon:Warn(minutes)
	local numGuild, numParty, numRaid = _G.GetNumGuildMembers(), _G.GetNumSubgroupMembers(), _G.GetNumGroupMembers()
	local msg = "[Broker_Wintergrasper] "

	if profileDB.soundAlert then
		_G.PlaySoundFile("Interface\\AddOns\\Broker_Wintergrasper\\Media\\brass_fanfare_4.mp3");
	end

	if minutes == 0 then
		msg = msg .. L["The battle for Wintergrasp has begun."]
	else
		msg = msg .. self:GetBattleTypeText() .. " " .. minutes .. L[" minutes."]
	end

	if profileDB.tellSelf then
		_G.DEFAULT_CHAT_FRAME:AddMessage("|cffff0000" .. msg .. "|r")
	end
	if profileDB.tellGuild and (numGuild > 0) then
		_G.SendChatMessage(msg,"GUILD",nil,nil)
	end
	if profileDB.tellParty and (numParty > 0) then
		_G.SendChatMessage(msg,"PARTY",nil,nil)
	end
	if profileDB.tellRaid and (numRaid > 0) then
		_G.SendChatMessage(msg,"RAID",nil,nil)
	end
end
